%{
  /* This is the prologue section. This code goes
  on the top of the parser implementation file. */
  #include <stdio.h>
  extern int yylex();
  extern int yyerror();
%}
%debug
/*
-----Declarations of terminals-----
Every time we create a token by using the %token
keyword, implicitly we are defining an integer constant
in the .h file. This .h file will be included
in the scanner file, in order to access those
symbols and possibly return them to the parser
*/
%token  INT DOUBLE ID PRINT IF WHILE ELSE
        RO RC BO BC SC CM SO S PLUS MINUS
        STAR DIV MIN MAJ MIN_EQ EQ_MIN MAJ_EQ
        EQ_MAJ EQ AND OR NOT INT_TYPE DOUBLE_TYPE
%%
/*
-----Grammar rules-----
The syntax for these rules is the following
left_hand_side: right_hand_side;

You can also specify more than one rule with the
same left_hand_side by using the following syntax
left_hand_side: right_hand_side_1 | right_end_side_2;

If you want to define an action to be run
whenever the rule is triggered, the syntax is
left_hand_side: right_hand_side { C code };

Remark that if you are defining more than one rule with
the | notation and you want an action to be run for each
of those rules, you need to supply an action for each of them.
left_hand_side: right_hand_side_1 {action 1}
              | right_hand_side_2 {action 2};
*/

prog : decl_list stmt_list {printf("Programm correctly recognized\n");}
;


//////////////////////////////////////
// Declarations
//////////////////////////////////////

decl_list : decl_list decl | 
;

decl : type var_list S
;

type : INT_TYPE | DOUBLE_TYPE
;

var_list : var | var_list CM var
;

var : ID array
;

array : | array SO INT SC
;


//////////////////////////////////////
// Instructions
//////////////////////////////////////

stmt_list : stmt_list stmt | stmt 
;


stmt : if | while | assignment | print | BO stmt_list BC
;

// Assignment instruction
assignment : id SC | id EQ exp S
;

// PRINT instruction
print : PRINT RO id RC S
;


// IF instruction
if : IF if_condition stmt ELSE stmt
     | IF if_condition stmt
;

if_condition : RO exp RC
;


// WHILE instruction
while : WHILE while_condition stmt
;

while_condition : RO exp RC
;


// Expressions
exp :
    /* Espressioni logiche */
    exp AND exp
    | exp OR exp
    | NOT exp

    /* Espressioni di confronto */
    | exp EQ EQ exp
    | exp MIN exp
    | exp MAJ exp
    | exp mineq exp
    | exp mageq exp

    /* Espressioni aritmetiche */
    | exp PLUS exp
    | exp MINUS exp
    | exp STAR exp
    | exp DIV exp
    | RO exp RC
    | id
    | INT
    | DOUBLE
    | MINUS INT 
    | MINUS DOUBLE 
;


mineq : MIN_EQ | EQ_MIN;
mageq : MAJ_EQ | EQ_MAJ;


id : ID
      |ID SO INT SC
      |ID SO ID SC
;