%option header-file="lex.yy.h"
%option noyywrap

%top{
  #include "parser.tab.h"
}

nl \r|\n|\r\n
ws [ \t]
id [A-Za-z_][A-Za-z0-9_]*
integer ([1-9][0-9]*|0)
double (([0-9]+\.[0-9]*)|([0-9]*\.[0-9]+))(e|E('+'|'-')?[0-9]+)?
COMMENT "/*"([^*]*|"*"+[^*/])"*"+\/

%%

"(" {return RO;}
")" {return RC;}
"{" {return BO;}
"}" {return BC;}
"=" {return EQ;}
"+" {return PLUS;}
"-" {return MINUS;}
"*" {return STAR;}
"/" {return DIV;}
"<" {return MIN;}
">" {return MAJ;}
"<=" {return MIN_EQ;}
"=<" {return EQ_MIN;}
">=" {return MAJ_EQ;}
"=>" {return EQ_MAJ;}
"&" {return AND;}
"|" {return OR;}
"!" {return NOT;}

"[" {return SO;}
"]" {return SC;}

"int" {return INT_TYPE;}
"double" {return DOUBLE_TYPE;}

print {return PRINT;}
if {return IF;}
while {return WHILE;}
else {return ELSE;}
; {return S;}
, {return CM;}

{id} {return ID;}
{integer} {return INT;}
{double} {return DOUBLE;}

{COMMENT} {;}

{ws}|{nl} {;}

. {printf("SCANNER ERROR: %s\n",yytext);}
